unit Resolutions;
{<Petit> composant permettant de changer dynamiquement de rsolution
  l'excution comme  la conception.
 @Copyright V1.3 Sylvain JAMES 01/10/1999 <ioulia@club-internet.fr>
 Remerciements express  Olivier Dahan http://perso.cybercable.fr/stargate/
 pour ses conseils d'expert :-)}

interface
{$DEFINE FRANCAIS} // possible : ENGLISH

uses Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, DsgnIntf;

type TModesProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValues(Proc: TGetStrProc); override;
  end;

type EResolution = class(Exception)
  private
    FELastError: string;
    FELastErrorNumber: integer;
  public
    property LastError: string read FELastError write FELastError;
    property LastErrorNumber: integer read FELastErrorNumber write FELastErrorNumber;
  end;

type TSJResolution = class(TComponent) //Hrite de TComponent
  private
//Pour la rsolution d'cran-------
    InitialMode: TDevMode;
    FBits: integer;    //Nb de bits
    FHRes: integer;    //Rsolution horizontale
    FVRes: integer;    //Rsolution verticale
    FBitsMEM: Integer; //  "
    FHResMEM: Integer; //Mme chose mais mmoris en permanence
    FVResMEM: Integer; //  "
    FMode: string;
    FRaiseException: boolean;
    FLastError: string;
    FLastErrorNumber: integer;
    FListModes : TStrings;
    MemoryDone: boolean;
    //Events
    FOnChangeMode : TNotifyEvent;  // OnChangeMode
    FOnRestoreMode : TNotifyEvent; // OnRestoreMode
    procedure Get_Resolutions;
    procedure Memory_Resolution;
    procedure SetFMode(value: string);
    procedure ParseMode(var W: integer; var H: integer; var B: integer; M: string);
    procedure Triggers_Exception;

  protected
    procedure Event_OnChangeMode;
    procedure Event_OnRestoreMode;

  public
    property LastError: string read FLastError;
    property LastErrorNumber: integer read FLastErrorNumber;
    property ListModes : TStrings read FListModes;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Set_Resolution(Width: integer; Height: integer; Bits: integer): boolean;
    procedure Restore_Resolution;

  published
    property Mode: string read FMode write SetFMode;
    property RaiseOnException: boolean read FRaiseException write FRaiseException;
    property OnChangeMode : TNotifyEvent read FOnchangeMode write FOnchangeMode;
    property OnRestoreMode : TNotifyEvent read FOnRestoreMode write FOnRestoreMode;
  end;

procedure register;

implementation

const
{$IFDEF ENGLISH}
  kBadMode = 'Bad mode';
  kRestart = 'You need Restart';
  kBadFlag = 'Bad flags';
  kFailed = 'Change resolution failed';
  kNotUpdated = 'Not updated';
  kParseMode = 'Attention : string mode need format ''HRes x VRes n Bit''. Spaces between resolutions aren''t obligatory but it need one between VRES and number of bit.';
{$ENDIF}

{$IFDEF FRANCAIS}
  kBadMode = 'Mode inconnu';
  kRestart = 'Il faut redmarrer';
  kBadFlag = 'Mauvais drapeaux';
  kFailed = 'Echec du changement de rsolution';
  kNotUpdated = 'Pas mis  jour';
  kParseMode = 'Attention : Le format chaine du mode doit tre ''HRes x VRes n Bits''. Les espaces entre les rsolutions ne sont pas obligatoires. Par contre, il en faut un entre VRes et le nombre de bits.';
{$ENDIF}

var
  Modes: array[0..255] of TDevMode;
  ModeNum: LongInt; //Nb de modes dispo

constructor TSJResolution.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  MemoryDone := false; //Rsolution encore non mmorise  la cration
  FListModes := TStringList.Create;
  Memory_Resolution;
end;

destructor TSJResolution.Destroy;
begin
  FListModes.Free;
  inherited Destroy;
end;

procedure TSJResolution.Memory_Resolution;
//Mmoriser la rsolution en cours et les autres disponibles
var
  DC: THandle;
begin
  DC := GetDC(GetDesktopWindow); {C'tait le coup de baguette magique...}
  FBitsMEM := GetDeviceCaps(DC, BITSPIXEL);
  FHResMEM := GetDeviceCaps(DC, HORZRES);
  FVResMEM := GetDeviceCaps(DC, VERTRES);
  MemoryDone := true;
  Get_Resolutions;
end;

procedure TSJResolution.Restore_Resolution;
//Restituer la rsolution mmorise initialement avec Memory_Resolution
begin
  if not MemoryDone then exit;
  ChangeDisplaySettings(InitialMode, 0);
  Event_OnRestoreMode;
end;

procedure TSJResolution.Get_Resolutions;
var
  DM: TDevMode; //
  Ok: Bool;
begin
  if not MemoryDone then Memory_Resolution;

  ModeNum := -1;
  repeat
    Inc(ModeNum);
    Ok := EnumDisplaySettings(nil, ModeNum, DM);
    if Ok = true then
      begin
        Modes[ModeNum] := DM;
        FListModes.Add(inttostr(DM.dmPelsWidth)+'x'+inttostr(DM.dmPelsHeight)+
                      ' '+inttostr(DM.dmBitsPerPel)+' b');
        if (FHResMEM = DM.dmPelsWidth) and (FVResMEM = DM.dmPelsHeight)
          and (FBitsMEM = DM.dmBitsPerPel) then
          begin
            InitialMode := Modes[ModeNum];
            FHRes := FHResMEM;
            FVRes := FVResMEM;
            FBits := FBitsMEM;
            FMode := inttostr(FHRes) + 'x' + inttostr(FVRes) + ' ' + inttostr(FBits) + ' b';
          end;
     end;
  until Ok = false;
end;

function TSJResolution.Set_Resolution(Width: integer; Height: integer; Bits: integer): boolean;
var
  Found: boolean; //Si on trouve le mode correspondant aux paramtres
  bcl, change: integer;
  NewMode: TDevMode;

begin
  Result := false;

  if (FHRes = Width) and (FVRes = Height) and (FBits = Bits) then
  //Si la rsolution demande est celle en cours, alors shooter
    begin
      Result := true;
      exit;
    end;

  Found := false;
  for bcl := 0 to ModeNum - 1 do
    begin
      NewMode := TDevMode(Modes[bcl]);
      if (Width = NewMode.dmPelsWidth) and (Height = NewMode.dmPelsHeight)
        and (Bits = NewMode.dmBitsPerPel) then
        begin
          Found := true;
          break;
        end;
    end;

  if not Found then begin
      FLastError := kBadMode;
      FLastErrorNumber := DISP_CHANGE_BADMODE;
      result := false;
      if RaiseOnException then Triggers_Exception;
      exit;
    end;

  NewMode.dmDisplayFrequency := 0;
  NewMode.dmDisplayFlags :=
    DM_BITSPERPEL and
    DM_PELSWIDTH and
    DM_PELSHEIGHT and
    DM_DISPLAYFLAGS;

  Change := ChangeDisplaySettings(NewMode, 0);
  FLastErrorNumber := Change;

  case Change of
    DISP_CHANGE_SUCCESSFUL:
      begin
        FLastError := '';
        FHRes := Width;
        FVRes := Height;
        FBits := Bits;
        result := true;
        Event_OnChangeMode;
      end;
    DISP_CHANGE_RESTART:
      begin
        FLastError := kRestart;
        result := false;
      end;
    DISP_CHANGE_BADFLAGS:
      begin
        FLastError := kBadFlag;
        result := false;
      end;
    DISP_CHANGE_FAILED:
      begin
        FLastError := kFailed;
        result := false;
      end;
    DISP_CHANGE_BADMODE:
      begin
        FLastError := kBadMode;
        result := false;
      end;
    DISP_CHANGE_NOTUPDATED:
      begin
        FLastError := kNotUpdated;
        result := false;
      end;
    else
      result := false;
  end;

  if RaiseOnException and (result = false) then Triggers_Exception;
end;

procedure TSJResolution.Triggers_Exception;
var
  e: EResolution;
begin
  e := EResolution.Create(FLastError);
 {Copie des variables d'erreur vers la classe d'exception EResolution
  Le premier membre est une proprit de EResolution, le deuxime une variable private de TSJResolution}
  e.LastError := FLastError;
  e.LastErrorNumber := FLastErrorNumber;
  //Dclenchement de l'exception
  raise e;
end;

procedure TSJResolution.SetFMode(value: string);
var
  W, H, B: integer;
begin
  if value <> FMode then
    begin
      ParseMode(W, H, B, value);
      if Set_Resolution(W, H, B) then
        FMode := inttostr(W) + 'x' + inttostr(H) + ' ' + inttostr(B) + ' b';
    end;
end;

procedure TSJResolution.ParseMode(var W: integer; var H: integer; var B: integer; M: string);
{Pour extraire d'une chaine ex. : '800x600 24 bits' les informations numriques}
var
  p1, p2, p3: integer;
begin
  M := trim(M);
  try
    p1 := pos('x', M);
    W := strtoint(trim(copy(M, 1, p1 - 1))); //Dtermination de Width
    p2 := pos(' ', M);
    H := strtoint(trim(copy(M, p1 + 1, p2 - p1 - 1))); //Height
    p3 := pos('b', M);
    B := strtoint(trim(copy(M, p2 + 1, p3 - p2 - 1))); //Bits

    if ((p1 = 0) or (p2 = 0) or (p3 = 0)) and RaiseOnException then
      begin
        FLastError := kParseMode;
        Triggers_Exception;
      end;

  except
    on e: exception do
      begin
        FLastError := e.Message + kParseMode;
        if RaiseOnException then Triggers_Exception;
      end;
  end;
end;

procedure TSJResolution.Event_OnChangeMode;
begin
  if Assigned(OnChangeMode) then OnChangeMode(Self);
end;

procedure TSJResolution.Event_OnRestoreMode;
begin
  if Assigned(OnRestoreMode) then OnRestoreMode(self);
end;

//Editeur de la proprit 'Mode'
function TModesProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList, paSortList, paReadOnly];
end;

procedure TModesProperty.GetValues(Proc: TGetStrProc);
var
  bcl: integer;
  begin
  for bcl := 0 to ModeNum - 1 do
    begin
      proc(inttostr(Modes[bcl].dmPelsWidth) + 'x' + inttostr(Modes[bcl].dmPelsHeight) +
        ' ' + inttostr(Modes[bcl].dmBitsPerPel) + ' b');
    end;
end;

procedure register;
begin
  RegisterComponents('Sylvain', [TSJResolution]);
  RegisterPropertyEditor(TypeInfo(string), TSJResolution, 'Mode', TModesProperty);
end;
end.

